#!/bin/bash

# Get the directory where this script is located
SCRIPT_DIR="$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )"
PROJECT_DIR="$( cd "$SCRIPT_DIR/.." && pwd )"

cd "$SCRIPT_DIR"

# Clean up old test data and any existing binaries
# Force removal with chmod to handle permission issues
if [ -d dehydrate_test_data ]; then
    chmod -R u+w dehydrate_test_data 2>/dev/null || true
    rm -rf dehydrate_test_data
fi
rm -f dehydrate_test

# Build the test program using Docker
# The Docker container supports both amd64 and arm64 architectures
docker run --rm \
    -v "$PROJECT_DIR":/workdir \
    -w /workdir/test \
    gitea.jde.nz/public/dropshell-build-base:latest \
    bash -c "
        # Verify we can find the source file
        if [ ! -f dehydrate_test.cpp ]; then
            echo 'ERROR: dehydrate_test.cpp not found in current directory'
            echo 'Working directory:' && pwd
            echo 'Available files:' && ls -la
            exit 1
        fi
        
        # Clean any existing binary and compile
        rm -f dehydrate_test
        if ! g++ -std=c++23 -static dehydrate_test.cpp -o dehydrate_test; then
            echo 'ERROR: Compilation failed'
            exit 1
        fi
        
        # Verify binary was created and is executable
        if [ ! -f dehydrate_test ]; then
            echo 'ERROR: Binary was not created'
            exit 1
        fi
        
        # Quick architecture check - just verify the binary format
        if ! file dehydrate_test | grep -q 'executable'; then
            echo 'ERROR: Generated file is not an executable'
            file dehydrate_test
            exit 1
        fi
    "

# Check if compilation succeeded
if [ ! -f "./dehydrate_test" ]; then
    echo "Error: Failed to compile dehydrate_test - binary not found"
    echo "Files in current directory:"
    ls -la
    exit 1
fi

# Fix ownership of the binary (created by Docker as root)
if [ "$(stat -c %u ./dehydrate_test)" != "$(id -u)" ]; then
    # Use Docker to change ownership to current user
    docker run --rm -v "$PROJECT_DIR":/workdir -w /workdir/test \
        gitea.jde.nz/public/dropshell-build-base:latest \
        chown "$(id -u):$(id -g)" dehydrate_test
fi

# Run the test
./dehydrate_test